@@ -1,6 +1,8 @@
 package Mojo::IRC::UA;
 use Mojo::Base 'Mojo::IRC';
 
+use IRC::Utils ();
+
 has op_timeout => 10;
 
 sub channels {
@@ -147,6 +149,42 @@ sub join_channel {
   );
 }
 
+sub kick {
+  my ($self, $command, $cb) = @_;
+  my ($target, $user) = $command =~ /(\S+)\s+(.*)/;
+  my $res = {reason => ''};
+
+  $target //= '';
+  $user   //= '';
+  $self->_write_and_wait(
+    Parse::IRC::parse_irc("KICK $command"),
+    {
+      err_needmoreparams   => {},
+      err_nosuchchannel    => {1 => $target},
+      err_nosuchnick       => {1 => $user},
+      err_badchanmask      => {1 => $target},
+      err_chanoprivsneeded => {1 => $target},
+      err_usernotinchannel => {1 => $user},
+      err_notonchannel     => {1 => $target},
+      irc_kick             => {0 => $target, 1 => $user},
+    },
+    sub {
+      my ($self, $event, $err, $msg) = @_;
+      my ($nick) = IRC::Utils::parse_user($msg->{prefix});
+      $msg->{params}[2] //= '';
+      $res->{reason} = $msg->{params}[2] eq $nick ? '' : $msg->{params}[2];
+      $self->$cb($event =~ /^err_/ ? $err || $msg->{params}[2] || $event : '', $res);
+    }
+  );
+}
+
+sub mode {
+  my $cb = pop;
+  my ($self, $mode) = @_;
+  return $self->_mode_for_channel($1, $2, $cb) if $mode and $mode =~ /(\S+)\s+(.+)/;
+  return $self->_mode_for_user($mode, $cb);    # get or set
+}
+
 sub nick {
   my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
   my ($self, $nick) = @_;
@@ -266,6 +304,56 @@ sub whois {
   );
 }
 
+sub _mode_for_channel {
+  my ($self, $target, $mode, $cb) = @_;
+  my $res = {banlist => [], exceptlist => [], invitelist => [], uniqopis => [], mode => $mode, params => ''};
+
+  return $self->_write_and_wait(
+    Parse::IRC::parse_irc("MODE $target $mode"),
+    {
+      err_chanoprivsneeded    => {1 => $target},
+      err_keyset              => {1 => $target},
+      err_needmoreparams      => {1 => $target},
+      err_nochanmodes         => {1 => $target},
+      err_unknownmode         => {1 => $target},
+      err_usernotinchannel    => {1 => $target},
+      irc_mode                => {0 => $target},
+      irc_rpl_endofbanlist    => {1 => $target},
+      irc_rpl_endofexceptlist => {1 => $target},
+      irc_rpl_endofinvitelist => {1 => $target},
+      irc_rpl_channelmodeis => sub { @$res{qw(mode params)} = @{$_[1]->{params}}[1, 2] },
+      irc_rpl_banlist    => sub { push @{$res->{banlist}},    $_[1]->{params}[1] },
+      irc_rpl_exceptlist => sub { push @{$res->{exceptlist}}, $_[1]->{params}[1] },
+      irc_rpl_invitelist => sub { push @{$res->{invitelist}}, $_[1]->{params}[1] },
+      irc_rpl_uniqopis   => sub { push @{$res->{uniqopis}},   $_[1]->{params}[1] },
+    },
+    sub {
+      my ($self, $event, $err, $msg) = @_;
+      $self->$cb($event =~ /^(?:err_)/ ? $err || $msg->{params}[2] || $event : '', $res);
+    }
+  );
+}
+
+sub _mode_for_user {
+  my ($self, $mode, $cb) = @_;
+  my $nick = $self->nick;
+
+  return $self->_write_and_wait(
+    Parse::IRC::parse_irc($mode ? "MODE $nick $mode" : "MODE $nick"),
+    {
+      err_umodeunknownflag => {},
+      err_needmoreparams   => {},
+      err_usersdontmatch   => {},
+      irc_mode             => {0 => $nick},
+      irc_rpl_umodeis      => {0 => $nick},
+    },
+    sub {
+      my ($self, $event, $err, $msg) = @_;
+      $self->$cb($event =~ /^(?:err_)/ ? $err || $event : '', $msg->{params}[1]);
+    }
+  );
+}
+
 sub _parse_namreply {
   my ($self, $msg, $users) = @_;
 
@@ -430,6 +518,24 @@ NOTE! This method will fail if the channel is already joined. Unfortunately,
 the way it will fail is simply by not calling the callback. This should be
 fixed - Just don't know how yet.
 
+=head2 kick
+
+  $self = $self->kick("#channel superman", sub { my ($self, $err, $res) = @_; });
+
+Used to kick a user. C<$res> looks like this:
+
+  {reason => "you don't behave"}
+
+=head2 mode
+
+  $self = $self->mode(sub { my ($self, $err, $mode) = @_; });
+  $self = $self->mode("-i", sub { my ($self, $err, $mode) = @_; });
+  $self = $self->mode("#channel +k secret", sub { my ($self, $err, $mode) = @_; });
+
+This method is used to get or set a user mode or set a channel mode.
+
+C<$mode> is EXPERIMENTAL, but holds a hash, with "mode" as key.
+
 =head2 nick
 
   $self = $self->nick($nick => sub { my ($self, $err) = @_; });
